Module: RSpec::Matchers

Included in:
DSL::Matcher
Defined in:
lib/rspec/matchers.rb,
lib/rspec/matchers.rb,
lib/rspec/matchers/dsl.rb,
lib/rspec/matchers/pretty.rb,
lib/rspec/matchers/matcher.rb,
lib/rspec/matchers/built_in.rb,
lib/rspec/matchers/be_close.rb,
lib/rspec/matchers/built_in/be.rb,
lib/rspec/matchers/built_in/eq.rb,
lib/rspec/matchers/built_in/eql.rb,
lib/rspec/matchers/built_in/has.rb,
lib/rspec/matchers/configuration.rb,
lib/rspec/matchers/built_in/have.rb,
lib/rspec/matchers/built_in/exist.rb,
lib/rspec/matchers/built_in/equal.rb,
lib/rspec/matchers/built_in/yield.rb,
lib/rspec/matchers/method_missing.rb,
lib/rspec/matchers/built_in/cover.rb,
lib/rspec/matchers/built_in/match.rb,
lib/rspec/matchers/built_in/change.rb,
lib/rspec/matchers/built_in/satisfy.rb,
lib/rspec/matchers/operator_matcher.rb,
lib/rspec/matchers/built_in/include.rb,
lib/rspec/matchers/built_in/be_within.rb,
lib/rspec/matchers/built_in/respond_to.rb,
lib/rspec/matchers/built_in/be_kind_of.rb,
lib/rspec/matchers/built_in/match_array.rb,
lib/rspec/matchers/built_in/raise_error.rb,
lib/rspec/matchers/built_in/throw_symbol.rb,
lib/rspec/matchers/built_in/base_matcher.rb,
lib/rspec/matchers/generated_descriptions.rb,
lib/rspec/matchers/built_in/be_instance_of.rb,
lib/rspec/matchers/built_in/start_and_end_with.rb,
lib/rspec/matchers/extensions/instance_eval_with_args.rb

Overview

RSpec::Matchers provides a number of useful matchers we use to compose expectations. A matcher is any object that responds to the following:

matches?(actual)
failure_message_for_should

These methods are also part of the matcher protocol, but are optional:

does_not_match?(actual)
failure_message_for_should_not
description

Predicates

In addition to matchers that are defined explicitly, RSpec will create custom matchers on the fly for any arbitrary predicate, giving your specs a much more natural language feel.

A Ruby predicate is a method that ends with a "?" and returns true or false. Common examples are empty?, nil?, and instance_of?.

All you need to do is write should be_ followed by the predicate without the question mark, and RSpec will figure it out from there. For example:

[].should be_empty     # => [].empty?() | passes
[].should_not be_empty # => [].empty?() | fails

In addtion to prefixing the predicate matchers with "be", you can also use "be_a" and "bean", making your specs read much more naturally:

"a string".should be_an_instance_of(String) =>"a string".instance_of?(String) #passes

3.should be_a_kind_of(Fixnum)        # => 3.kind_of?(Numeric)     | passes
3.should be_a_kind_of(Numeric)       # => 3.kind_of?(Numeric)     | passes
3.should be_an_instance_of(Fixnum)   # => 3.instance_of?(Fixnum)  | passes
3.should_not be_instance_of(Numeric) # => 3.instance_of?(Numeric) | fails

RSpec will also create custom matchers for predicates like has_key?. To use this feature, just state that the object should have_key(:key) and RSpec will call has_key?(:key) on the target. For example:

{:a => "A"}.should have_key(:a) # => {:a => "A"}.has_key?(:a) | passes
{:a => "A"}.should have_key(:b) # => {:a => "A"}.has_key?(:b) | fails

You can use this feature to invoke any predicate that begins with "has_", whether it is part of the Ruby libraries (like Hash#has_key?) or a method you wrote on your own class.

Custom Matchers

When you find that none of the stock matchers provide a natural feeling expectation, you can very easily write your own using RSpec's matcher DSL or writing one from scratch.

Matcher DSL

Imagine that you are writing a game in which players can be in various zones on a virtual board. To specify that bob should be in zone 4, you could say:

bob.current_zone.should eql(Zone.new("4"))

But you might find it more expressive to say:

bob.should be_in_zone("4")

and/or

bob.should_not be_in_zone("3")

You can create such a matcher like so:

RSpec::Matchers.define :be_in_zone do |zone|
  match do |player|
    player.in_zone?(zone)
  end
end

This will generate a be_in_zone method that returns a matcher with logical default messages for failures. You can override the failure messages and the generated description as follows:

RSpec::Matchers.define :be_in_zone do |zone|
  match do |player|
    player.in_zone?(zone)
  end

  failure_message_for_should do |player|
    # generate and return the appropriate string.
  end

  failure_message_for_should_not do |player|
    # generate and return the appropriate string.
  end

  description do
    # generate and return the appropriate string.
  end
end

Each of the message-generation methods has access to the block arguments passed to the create method (in this case, zone). The failure message methods (failure_message_for_should and failure_message_for_should_not) are passed the actual value (the receiver of should or should_not).

Custom Matcher from scratch

You could also write a custom matcher from scratch, as follows:

class BeInZone
  def initialize(expected)
    @expected = expected
  end

  def matches?(target)
    @target = target
    @target.current_zone.eql?(Zone.new(@expected))
  end

  def failure_message_for_should
    "expected #{@target.inspect} to be in Zone #{@expected}"
  end

  def failure_message_for_should_not
    "expected #{@target.inspect} not to be in Zone #{@expected}"
  end
end

... and a method like this:

def be_in_zone(expected)
  BeInZone.new(expected)
end

And then expose the method to your specs. This is normally done by including the method and the class in a module, which is then included in your spec:

module CustomGameMatchers
  class BeInZone
    # ...
  end

  def be_in_zone(expected)
    # ...
  end
end

describe "Player behaviour" do
  include CustomGameMatchers
  # ...
end

or you can include in globally in a spec_helper.rb file required from your spec file(s):

RSpec::configure do |config|
  config.include(CustomGameMatchers)
end

Defined Under Namespace

Modules: BuiltIn, DSL, Extensions, Pretty Classes: Configuration, OperatorMatcher

Class Attribute Summary (collapse)

Class Method Summary (collapse)

Instance Method Summary (collapse)

Dynamic Method Handling

This class handles dynamic methods through the method_missing method

- (Object) method_missing(method, *args, &block) (private)



6
7
8
9
10
# File 'lib/rspec/matchers/method_missing.rb', line 6

def method_missing(method, *args, &block)
  return Matchers::BuiltIn::BePredicate.new(method, *args, &block) if method.to_s =~ /^be_/
  return Matchers::BuiltIn::Has.new(method, *args, &block) if method.to_s =~ /^have_/
  super
end

Class Attribute Details

+ (Object) last_matcher

Returns the value of attribute last_matcher



4
5
6
# File 'lib/rspec/matchers/generated_descriptions.rb', line 4

def last_matcher
  @last_matcher
end

+ (Object) last_should

Returns the value of attribute last_should



4
5
6
# File 'lib/rspec/matchers/generated_descriptions.rb', line 4

def last_should
  @last_should
end

Class Method Details

+ (Object) clear_generated_description



7
8
9
10
# File 'lib/rspec/matchers/generated_descriptions.rb', line 7

def self.clear_generated_description
  self.last_matcher = nil
  self.last_should = nil
end

+ (RSpec::Matchers::Configuration) configuration

The configuration object

Returns:



100
101
102
# File 'lib/rspec/matchers/configuration.rb', line 100

def self.configuration
  @configuration ||= Configuration.new
end

+ (Object) generated_description



12
13
14
15
# File 'lib/rspec/matchers/generated_descriptions.rb', line 12

def self.generated_description
  return nil if last_should.nil?
  "#{last_should.to_s.gsub('_',' ')} #{last_description}"
end

Instance Method Details

- (Object) be(*args)

Given true, false, or nil, will pass if actual value is true, false or nil (respectively). Given no args means the caller should satisfy an if condition (to be or not to be).

Predicates are any Ruby method that ends in a "?" and returns true or false. Given be_ followed by arbitrary_predicate (without the "?"), RSpec will match convert that into a query against the target object.

The arbitrarypredicate feature will handle any predicate prefixed with "be_an" (e.g. bean_instance_of), "be_a" (e.g. bea_kind_of) or "be" (e.g. be_empty), letting you choose the prefix that best suits the predicate.

Examples:

actual.should be_true
actual.should be_false
actual.should be_nil
actual.should be_[arbitrary_predicate](*args)
actual.should_not be_nil
actual.should_not be_[arbitrary_predicate](*args)


227
228
229
230
# File 'lib/rspec/matchers.rb', line 227

def be(*args)
  args.empty? ?
    Matchers::BuiltIn::Be.new : equal(*args)
end

- (Object) be_a(klass) Also known as: be_an

passes if target.kind_of?(klass)



233
234
235
# File 'lib/rspec/matchers.rb', line 233

def be_a(klass)
  be_a_kind_of(klass)
end

- (Object) be_a_kind_of(expected) Also known as: be_kind_of

Passes if actual.kind_of?(expected)

Examples:


5.should be_kind_of(Fixnum)
5.should be_kind_of(Numeric)
5.should_not be_kind_of(Float)


259
260
261
# File 'lib/rspec/matchers.rb', line 259

def be_a_kind_of(expected)
  BuiltIn::BeAKindOf.new(expected)
end

- (Object) be_an_instance_of(expected) Also known as: be_instance_of

Passes if actual.instance_of?(expected)

Examples:


5.should be_instance_of(Fixnum)
5.should_not be_instance_of(Numeric)
5.should_not be_instance_of(Float)


246
247
248
# File 'lib/rspec/matchers.rb', line 246

def be_an_instance_of(expected)
  BuiltIn::BeAnInstanceOf.new(expected)
end

- (Object) be_close(expected, delta)

Deprecated.

use +be_within+ instead.



4
5
6
7
# File 'lib/rspec/matchers/be_close.rb', line 4

def be_close(expected, delta)
  RSpec.deprecate("be_close(#{expected}, #{delta})", "be_within(#{delta}).of(#{expected})")
  be_within(delta).of(expected)
end

- (Object) be_false

Passes if actual is falsy (false or nil)



198
199
200
# File 'lib/rspec/matchers.rb', line 198

def be_false
  BuiltIn::BeFalse.new
end

- (Object) be_nil

Passes if actual is nil



203
204
205
# File 'lib/rspec/matchers.rb', line 203

def be_nil
  BuiltIn::BeNil.new
end

- (Object) be_true

Passes if actual is truthy (anything but false or nil)



193
194
195
# File 'lib/rspec/matchers.rb', line 193

def be_true
  BuiltIn::BeTrue.new
end

- (Object) be_within(delta)

Passes if actual == expected +/- delta

Examples:


result.should be_within(0.5).of(3.0)
result.should_not be_within(0.5).of(3.0)


271
272
273
# File 'lib/rspec/matchers.rb', line 271

def be_within(delta)
  BuiltIn::BeWithin.new(delta)
end

- (Object) change(receiver = nil, message = nil, &block)

Applied to a proc, specifies that its execution will cause some value to change.

You can either pass receiver and message, or a block, but not both.

When passing a block, it must use the { ... } format, not do/end, as { ... } binds to the +change+ method, whereas do/end would errantly bind to the +should+ or +should_not+ method.

== Notes

Evaluates receiver.message or block before and after it evaluates the proc object (generated by the lambdas in the examples above).

should_not change only supports the form with no subsequent calls to by, by_at_least, by_at_most, to or from.

Examples:


lambda {
  team.add_player(player)
}.should change(roster, :count)

lambda {
  team.add_player(player)
}.should change(roster, :count).by(1)

lambda {
  team.add_player(player)
}.should change(roster, :count).by_at_least(1)

lambda {
  team.add_player(player)
}.should change(roster, :count).by_at_most(1)

string = "string"
lambda {
  string.reverse!
}.should change { string }.from("string").to("gnirts")

lambda {
  person.happy_birthday
}.should change(person, :birthday).from(32).to(33)

lambda {
  employee.develop_great_new_social_networking_app
}.should change(employee, :title).from("Mail Clerk").to("CEO")

lambda {
  doctor.leave_office
}.should change(doctor, :sign).from(/is in/).to(/is out/)

user = User.new(:type => "admin")
lambda {
  user.symbolize_type
}.should change(user, :type).from(String).to(Symbol)

Parameters:

  • receiver (Object) (defaults to: nil)
  • message (Symbol) (defaults to: nil)

    the message to send the receiver



337
338
339
# File 'lib/rspec/matchers.rb', line 337

def change(receiver=nil, message=nil, &block)
  BuiltIn::Change.new(receiver, message, &block)
end

- (Object) cover(*values)



353
354
355
# File 'lib/rspec/matchers.rb', line 353

def cover(*values)
  BuiltIn::Cover.new(*values)
end

- (Object) end_with(*expected)

Matches if the actual value ends with the expected value(s). In the case of a string, matches against the last expected.length characters of the actual string. In the case of an array, matches against the last expected.length elements of the actual array.

Examples:


"this string".should end_with "string"
[0, 1, 2, 3, 4].should end_with 4
[0, 2, 3, 4, 4].should end_with 3, 4


367
368
369
# File 'lib/rspec/matchers.rb', line 367

def end_with(*expected)
  BuiltIn::EndWith.new(*expected)
end

- (Object) eq(expected)

Passes if actual == expected.

See http://www.ruby-doc.org/core/classes/Object.html#M001057 for more information about equality in Ruby.

Examples:


5.should eq(5)
5.should_not eq(3)


379
380
381
# File 'lib/rspec/matchers.rb', line 379

def eq(expected)
  BuiltIn::Eq.new(expected)
end

- (Object) eql(expected)

Passes if +actual.eql?(expected)+

See http://www.ruby-doc.org/core/classes/Object.html#M001057 for more information about equality in Ruby.

Examples:


5.should eql(5)
5.should_not eql(3)


391
392
393
# File 'lib/rspec/matchers.rb', line 391

def eql(expected)
  BuiltIn::Eql.new(expected)
end

- (Object) equal(expected)

Passes if actual.equal?(expected) (object identity).

See http://www.ruby-doc.org/core/classes/Object.html#M001057 for more information about equality in Ruby.

Examples:


5.should equal(5) # Fixnums are equal
"5".should_not equal("5") # Strings that look the same are not the same object


403
404
405
# File 'lib/rspec/matchers.rb', line 403

def equal(expected)
  BuiltIn::Equal.new(expected)
end

- (Object) exist(*args)

Passes if actual.exist? or actual.exists?

Examples:

File.should exist("path/to/file")


411
412
413
# File 'lib/rspec/matchers.rb', line 411

def exist(*args)
  BuiltIn::Exist.new(*args)
end

- (Object) have(n) Also known as: have_exactly

Passes if receiver is a collection with the submitted number of items OR if the receiver OWNS a collection with the submitted number of items.

If the receiver OWNS the collection, you must use the name of the collection. So if a Team instance has a collection named #players, you must use that name to set the expectation.

If the receiver IS the collection, you can use any name you like for named_collection. We'd recommend using either "elements", "members", or "items" as these are all standard ways of describing the things IN a collection.

This also works for Strings, letting you set expectations about their lengths.

Examples:


# Passes if team.players.size == 11
team.should have(11).players

# Passes if [1,2,3].length == 3
[1,2,3].should have(3).items #"items" is pure sugar

# Passes if ['a', 'b', 'c'].count == 3
[1,2,3].should have(3).items #"items" is pure sugar

# Passes if "this string".length == 11
"this string".should have(11).characters #"characters" is pure sugar


443
444
445
# File 'lib/rspec/matchers.rb', line 443

def have(n)
  BuiltIn::Have.new(n)
end

- (Object) have_at_least(n)

Exactly like have() with >=.

Warning:

should_not have_at_least is not supported

Examples:

"this".should have_at_least(3).letters


456
457
458
# File 'lib/rspec/matchers.rb', line 456

def have_at_least(n)
  BuiltIn::Have.new(n, :at_least)
end

- (Object) have_at_most(n)

Exactly like have() with <=.

Warning:

should_not have_at_most is not supported

Examples:

should have_at_most(number).items


468
469
470
# File 'lib/rspec/matchers.rb', line 468

def have_at_most(n)
  BuiltIn::Have.new(n, :at_most)
end

- (Object) include(*expected)

Passes if actual includes expected. This works for collections and Strings. You can also pass in multiple args and it will only pass if all args are found in collection.

Examples:


[1,2,3].should include(3)
[1,2,3].should include(2,3) #would pass
[1,2,3].should include(2,3,4) #would fail
[1,2,3].should_not include(4)
"spread".should include("read")
"spread".should_not include("red")


484
485
486
# File 'lib/rspec/matchers.rb', line 484

def include(*expected)
  BuiltIn::Include.new(*expected)
end

- (Object) match(expected) Also known as: match_regex

Note:

Due to Ruby's method dispatch mechanism, using the #match matcher

Given a Regexp or String, passes if actual.match(pattern)

within a custom matcher defined via the matcher DSL (RSpec::Matcher.define) will result Ruby calling the wrong #match method and raising an ArgumentError. Instead, use the aliased #match_regex method.

Examples:


email.should match(/^([^\s]+)((?:[-a-z0-9]+\.)+[a-z]{2,})$/i)
email.should match("@example.com")
zipcode.should match_regex(/\A\d{5}(-\d{4})?\z/)
zipcode.should match_regex("90210")


502
503
504
# File 'lib/rspec/matchers.rb', line 502

def match(expected)
  BuiltIn::Match.new(expected)
end

- (Object) match_array(array)

Note:

This is also available using the =~ operator with should, but =~ is not supported with expect.

Note:

There is no should_not version of array.should =~ other_array

Passes if actual contains all of the expected regardless of order. This works for collections. Pass in multiple args and it will only pass if all args are found in collection.

Examples:


expect([1,2,3]).to match_array([1,2,3])
expect([1,2,3]).to match_array([1,3,2])
[1,2,3].should   =~ [1,2,3]   # => would pass
[1,2,3].should   =~ [2,3,1]   # => would pass
[1,2,3,4].should =~ [1,2,3]   # => would fail
[1,2,2,3].should =~ [1,2,3]   # => would fail
[1,2,3].should   =~ [1,2,3,4] # => would fail


693
694
695
# File 'lib/rspec/matchers.rb', line 693

def match_array(array)
  BuiltIn::MatchArray.new(array)
end

- (Object) raise_error(error = Exception, message = nil, &block) Also known as: raise_exception

With no args, matches if any error is raised. With a named error, matches only if that specific error is raised. With a named error and messsage specified as a String, matches only if both match. With a named error and messsage specified as a Regexp, matches only if both match. Pass an optional block to perform extra verifications on the exception matched

Examples:


lambda { do_something_risky }.should raise_error
lambda { do_something_risky }.should raise_error(PoorRiskDecisionError)
lambda { do_something_risky }.should raise_error(PoorRiskDecisionError) { |error| error.data.should == 42 }
lambda { do_something_risky }.should raise_error(PoorRiskDecisionError, "that was too risky")
lambda { do_something_risky }.should raise_error(PoorRiskDecisionError, /oo ri/)

lambda { do_something_risky }.should_not raise_error
lambda { do_something_risky }.should_not raise_error(PoorRiskDecisionError)
lambda { do_something_risky }.should_not raise_error(PoorRiskDecisionError, "that was too risky")
lambda { do_something_risky }.should_not raise_error(PoorRiskDecisionError, /oo ri/)


526
527
528
# File 'lib/rspec/matchers.rb', line 526

def raise_error(error=Exception, message=nil, &block)
  BuiltIn::RaiseError.new(error, message, &block)
end

- (Object) respond_to(*names)

Matches if the target object responds to all of the names provided. Names can be Strings or Symbols.



537
538
539
# File 'lib/rspec/matchers.rb', line 537

def respond_to(*names)
  BuiltIn::RespondTo.new(*names)
end

- (Object) satisfy(&block)

Passes if the submitted block returns true. Yields target to the block.

Generally speaking, this should be thought of as a last resort when you can't find any other way to specify the behaviour you wish to specify.

If you do find yourself in such a situation, you could always write a custom matcher, which would likely make your specs more expressive.

Examples:


5.should satisfy { |n|
  n > 3
}


556
557
558
# File 'lib/rspec/matchers.rb', line 556

def satisfy(&block)
  BuiltIn::Satisfy.new(&block)
end

- (Object) start_with(*expected)

Matches if the actual value starts with the expected value(s). In the case of a string, matches against the first expected.length characters of the actual string. In the case of an array, matches against the first expected.length elements of the actual array.

Examples:


"this string".should start_with "this s"
[0, 1, 2, 3, 4].should start_with 0
[0, 2, 3, 4, 4].should start_with 0, 1


570
571
572
# File 'lib/rspec/matchers.rb', line 570

def start_with(*expected)
  BuiltIn::StartWith.new(*expected)
end

- (Object) throw_symbol(expected_symbol = nil, expected_arg = nil)

Given no argument, matches if a proc throws any Symbol.

Given a Symbol, matches if the given proc throws the specified Symbol.

Given a Symbol and an arg, matches if the given proc throws the specified Symbol with the specified arg.

Examples:


lambda { do_something_risky }.should throw_symbol
lambda { do_something_risky }.should throw_symbol(:that_was_risky)
lambda { do_something_risky }.should throw_symbol(:that_was_risky, culprit)

lambda { do_something_risky }.should_not throw_symbol
lambda { do_something_risky }.should_not throw_symbol(:that_was_risky)
lambda { do_something_risky }.should_not throw_symbol(:that_was_risky, culprit)


590
591
592
# File 'lib/rspec/matchers.rb', line 590

def throw_symbol(expected_symbol=nil, expected_arg=nil)
  BuiltIn::ThrowSymbol.new(expected_symbol, expected_arg)
end

- (Object) yield_control

Note:

Your expect block must accept a parameter and pass it on to the method-under-test as a block.

Note:

This matcher is not designed for use with methods that yield multiple times.

Passes if the method called in the expect block yields, regardless of whether or not arguments are yielded.

Examples:


expect { |b| 5.tap(&b) }.to yield_control
expect { |b| "a".to_sym(&b) }.not_to yield_control


606
607
608
# File 'lib/rspec/matchers.rb', line 606

def yield_control
  BuiltIn::YieldControl.new
end

- (Object) yield_successive_args(*args)

Note:

Your expect block must accept a parameter and pass it on to the method-under-test as a block.

Designed for use with methods that repeatedly yield (such as iterators). Passes if the method called in the expect block yields multiple times with arguments matching those given.

Argument matching is done using === (the case match operator) and ==. If the expected and actual arguments match with either operator, the matcher will pass.

Examples:


expect { |b| [1, 2, 3].each(&b) }.to yield_successive_args(1, 2, 3)
expect { |b| { :a => 1, :b => 2 }.each(&b) }.to yield_successive_args([:a, 1], [:b, 2])
expect { |b| [1, 2, 3].each(&b) }.not_to yield_successive_args(1, 2)


672
673
674
# File 'lib/rspec/matchers.rb', line 672

def yield_successive_args(*args)
  BuiltIn::YieldSuccessiveArgs.new(*args)
end

- (Object) yield_with_args(*args)

Note:

Your expect block must accept a parameter and pass it on to the method-under-test as a block.

Note:

This matcher is not designed for use with methods that yield multiple times.

Given no arguments, matches if the method called in the expect block yields with arguments (regardless of what they are or how many there are).

Given arguments, matches if the method called in the expect block yields with arguments that match the given arguments.

Argument matching is done using === (the case match operator) and ==. If the expected and actual arguments match with either operator, the matcher will pass.

Examples:


expect { |b| 5.tap(&b) }.to yield_with_args # because #tap yields an arg
expect { |b| 5.tap(&b) }.to yield_with_args(5) # because 5 == 5
expect { |b| 5.tap(&b) }.to yield_with_args(Fixnum) # because Fixnum === 5
expect { |b| File.open("f.txt", &b) }.to yield_with_args(/txt/) # because /txt/ === "f.txt"

expect { |b| User.transaction(&b) }.not_to yield_with_args # because it yields no args
expect { |b| 5.tap(&b) }.not_to yield_with_args(1, 2, 3)


652
653
654
# File 'lib/rspec/matchers.rb', line 652

def yield_with_args(*args)
  BuiltIn::YieldWithArgs.new(*args)
end

- (Object) yield_with_no_args

Note:

Your expect block must accept a parameter and pass it on to the method-under-test as a block.

Note:

This matcher is not designed for use with methods that yield multiple times.

Passes if the method called in the expect block yields with no arguments. Fails if it does not yield, or yields with arguments.

Examples:


expect { |b| User.transaction(&b) }.to yield_with_no_args
expect { |b| 5.tap(&b) }.not_to yield_with_no_args # because it yields with `5`
expect { |b| "a".to_sym(&b) }.not_to yield_with_no_args # because it does not yield


623
624
625
# File 'lib/rspec/matchers.rb', line 623

def yield_with_no_args
  BuiltIn::YieldWithNoArgs.new
end